#include "RainDrop.h"
#include "RainLut.h"

RainDrop::RainDrop( int32 x,
						int32 y,
						int32 width,
						color_space space,
						BBitmap *oldScreen,
						BView *view,
						RainLut **lutArray,
						int32 numLuts )
	: mNextDrop( NULL ),
		mX( x ),
		mY( y ),
		mWidth( width ),
		mColorSpace( space ),
		mOldScreen( oldScreen ),
		mScreenView( view ),
		mLutArray( lutArray ),
		mNumLuts( numLuts ),
		mAlive( true ),
		mThread( -1 ),
		mMap( NULL )
{		
	BRect r( 0, 0, mWidth - 1, mWidth - 1 );
	
	mMap = new BBitmap( r, mColorSpace, true );
	
	return;
}

RainDrop::~RainDrop()
{
	delete mMap;

	return;
}

long RainDrop::DropThreadStart( void *data )
{
	long	result;

	result = ( (RainDrop*)data )->RealStart();

	return result;
}

long RainDrop::RealStart( void )
{
	for ( int32 phase = 0; mAlive && (phase < mNumLuts); phase++ ){
		const uint16 *lutPtr = mLutArray[ phase ]->GetLut();

		switch( mColorSpace ){
			case B_CMAP8:
				for ( int32 y = 0; y < mWidth; y++ ){
					uint8 *outPtr;
					uint8 *inPtr;
					
					outPtr = ((uint8*)mMap->Bits()) + ( mMap->BytesPerRow() * y );
					inPtr = ((uint8*)mOldScreen->Bits()) + mX +
								( mOldScreen->BytesPerRow() * ( y + mY ) );
								
					for ( int32 x = 0; x < mWidth; x++ ){
						*outPtr++ = *( inPtr++ + *lutPtr++ );
					}
				}
				break;
		
			case B_RGB15:
				for ( int32 y = 0; y < mWidth; y++ ){
					uint16 *outPtr;
					uint16 *inPtr;
					
					outPtr = ((uint16*)mMap->Bits()) + ( mMap->BytesPerRow() * y / 2);
					inPtr = ((uint16*)mOldScreen->Bits()) + mX +
								( mOldScreen->BytesPerRow() * ( y + mY ) / 2 );
								
					for ( int32 x = 0; x < mWidth; x++ ){
						*outPtr++ = *( inPtr++ + *lutPtr++ );
					}
				}
				break;
			case B_RGB32:
				for ( int32 y = 0; y < mWidth; y++ ){
					uint32 *outPtr;
					uint32 *inPtr;
					
					outPtr = ((uint32*)mMap->Bits()) + ( mMap->BytesPerRow() * y / 4);
					inPtr = ((uint32*)mOldScreen->Bits()) + mX +
								( mOldScreen->BytesPerRow() * ( y + mY ) / 4 );
								
					for ( int32 x = 0; x < mWidth; x++ ){
						*outPtr++ = *( inPtr++ + *lutPtr++ );
					}
				}
				break;
		}

		mScreenView->Window()->Lock();
		mScreenView->DrawBitmap( mMap, BPoint( mX, mY ) );
		mScreenView->Window()->Unlock();

		snooze( 5000 );
	}
	
	mAlive = false;
	
	return B_NO_ERROR;
}

long RainDrop::StartThread()
{
	mThread = spawn_thread( DropThreadStart,
											"A Drop",
											B_NORMAL_PRIORITY,
											(void*)(this) );
	if ( mThread >= B_NO_ERROR ){
		resume_thread( mThread );
	}
	
	return B_NO_ERROR;
}